/*
 *  Render.cpp
 *  Tcc
 */

#include "Render.h"
#include "vtkmsqAnalyzeReader.h"
#include "vtkmsqImageProperties.h"
#include <vtkVolumeRayCastMIPFunction.h>
#include "vtkVolumeRayCastMinIPFunction.h"
#include "vtkVolumeRayCastAveIPFunction.h"
#include "vtkVolumeRayCastMedianIPFunction.h"
#include <stdlib.h>
#include <QObject>
#include <QFileDialog>
#include <QMessageBox>
#include <QTableWidgetItem>
#include <QTimer>

#include <vtkColorTransferFunction.h>
#include <vtkRenderer.h>
#include <vtkRenderWindow.h>
#include <vtkInteractorStyleTrackballCamera.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkPiecewiseFunction.h>
#include <vtkVolumeProperty.h>
#include <vtkVolumeRayCastMapper.h>
#include <vtkOpenGLVolumeTextureMapper3D.h>
#include <vtkVolume.h>
#include <vtkCommand.h>
#include <vtkBoxWidget.h>
#include <vtkTransform.h>
#include <vtkEventQtSlotConnect.h>
#include <vtkSmartPointer.h>
#include <vtkImageReader.h>
#include <vtkImageData.h>
#include <vtkPointData.h>
#include <vtkLookupTable.h>
#include <vtkVolumeRayCastFunction.h>
#include <vtkLODProp3D.h>
#include <vtkCamera.h>
#include <vtkImageShiftScale.h>

#include <time.h>


using namespace std;

Render::Render()
{
	this->setupUi(this);
	this->properties = vtkmsqImageProperties::New();
	
	action_Open->setShortcuts(QKeySequence::Open);
	action_Open->setStatusTip(tr("Open an existing image file"));
	action_Open->setIconVisibleInMenu(false);
	connect(action_Open, SIGNAL(triggered()), this, SLOT(fileOpen()));

	action_Quit->setShortcuts(QKeySequence::Quit);
	action_Quit->setStatusTip(tr("Quit the program"));
	action_Quit->setIconVisibleInMenu(false);
	connect(action_Quit, SIGNAL(triggered()), qApp, SLOT(quit()));
	disableButtons();
	connect(CreateRGBA, SIGNAL(clicked()), this, SLOT(AddRGBAPoint()));
	connect(RemoveRGBA, SIGNAL(clicked()), this, SLOT(RemoveRGBAPoint()));
	connect(RGBATableWidget, SIGNAL(itemClicked(QTableWidgetItem *)), this, SLOT(updateRGBA()));
	connect(WhiteButton, SIGNAL(clicked()), this, SLOT(changeToWhite()));
	connect(BlackButton, SIGNAL(clicked()), this, SLOT(changeToBlack()));
	connect(ProjectionBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(changeRenderFunction()));
	connect(AnimationButton, SIGNAL(clicked()), this, SLOT(animation()));
	isRotating = false;
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(rotate()));
	connect(LODSlider, SIGNAL(valueChanged(int)), this, SLOT(changeLOD()));
}

void Render::changeLOD() {
	double value = LODSlider->value();
	interactor->SetStillUpdateRate(15 - 0.15*value);
	declarativeView->GetRenderWindow()->SetDesiredUpdateRate(15 -0.15*value);
	declarativeView->update();
}

void Render::animation() {
	if(isRotating) {
		timer->stop();
		isRotating = false;
		AnimationButton->setText(QApplication::translate("Layout", "Animation", 0, QApplication::UnicodeUTF8));
	} else {
		timer->start();
		isRotating = true;
		AnimationButton->setText(QApplication::translate("Layout", "Stop Animation", 0, QApplication::UnicodeUTF8));
	}
}

void Render::rotate() {
	int angle = DegreeSpinBox->value();
  // rotate the active camera by one degree
  renderer->GetActiveCamera()->Azimuth(angle);
		
  // render the image
  renderWindow->Render();
}

void Render::changeToAveIPFunction(int min, int max) {
	int mean = (min + max)/2;
	disableButtons();
 	rayCastFunction = vtkVolumeRayCastAveIPFunction::New();
	colorTransferFunction->AddRGBPoint(min, 1.0, 1.0, 1.0);
	colorTransferFunction->AddRGBPoint(max, 1.0, 1.0, 1.0);
	colorTransferFunction->AddRGBPoint(mean, 1.0, 0, 0);
	opacityTransferFunction->AddPoint(min, 0);
	opacityTransferFunction->AddPoint(max, 0);
	opacityTransferFunction->AddPoint(mean, 1);
 	volumeProperty->SetColor(colorTransferFunction);
 	volumeProperty->SetScalarOpacity(opacityTransferFunction);
	renderer->SetBackground(0, 0, 0);
	BlackButton->setChecked(true);
	configureButtons(min, max);
	insertRow(min, 255, 255, 255, 0);
	insertRow(max, 255, 255, 255, 0);
	insertRow(mean, 255, 0, 0, 100);
}

void Render::changeToMIPFunction(int min, int max) {
	disableButtons();
  rayCastFunction = vtkVolumeRayCastMIPFunction::New();
	opacityTransferFunction = vtkPiecewiseFunction::New();
  colorTransferFunction = vtkColorTransferFunction::New();
	opacityTransferFunction = vtkPiecewiseFunction::New();
	opacityTransferFunction->AddPoint(min, 0.0);
	opacityTransferFunction->AddPoint(max, 1.0);
	colorTransferFunction = vtkColorTransferFunction::New();
	colorTransferFunction->AddRGBPoint(min, 1.0, 1.0, 1.0);
	colorTransferFunction->AddRGBPoint(max, 0.0, 0.0, 0.0);
	
  volumeProperty->SetColor(colorTransferFunction);
  volumeProperty->SetScalarOpacity(opacityTransferFunction);
  volumeMapper->SetVolumeRayCastFunction(rayCastFunction);
	renderer->SetBackground(1.0, 1.0, 1.0);
	WhiteButton->setChecked(true);
	configureButtons(min, max);
	insertRow(min, 255, 255, 255, 0);
	insertRow(max, 0, 0, 0, 100);
}

void Render::changeToMinIPFunction(int min, int max) {
	int intensity = min + (max * 0.05);
	disableButtons();
  rayCastFunction = vtkVolumeRayCastMinIPFunction::New();
	colorTransferFunction->AddRGBPoint(min, 1.0, 0, 0);
	colorTransferFunction->AddRGBPoint(max, 0, 1.0, 0);
	colorTransferFunction->AddRGBPoint(intensity, 0, 0, 1.0);
	opacityTransferFunction->AddPoint(min, 0);
	opacityTransferFunction->AddPoint(max, 0);
	opacityTransferFunction->AddPoint(intensity, 1);
 	volumeProperty->SetColor(colorTransferFunction);
 	volumeProperty->SetScalarOpacity(opacityTransferFunction);
	renderer->SetBackground(0, 0, 0);
	BlackButton->setChecked(true);
	configureButtons(min, max);
	insertRow(min, 255, 0, 0, 0);
	insertRow(max, 0, 255, 0, 0);
	insertRow(intensity, 0, 0, 255, 100);
}

void Render::changeToMedianIPFunction(int min, int max) {
	int mean = (min + max)/2;
	disableButtons();
 	rayCastFunction = vtkVolumeRayCastMedianIPFunction::New();
	colorTransferFunction->AddRGBPoint(min, 1.0, 1.0, 1.0);
	colorTransferFunction->AddRGBPoint(max, 1.0, 1.0, 1.0);
	colorTransferFunction->AddRGBPoint(mean, 1.0, 0, 0);
	opacityTransferFunction->AddPoint(min, 0);
	opacityTransferFunction->AddPoint(max, 0);
	opacityTransferFunction->AddPoint(mean, 1);
 	volumeProperty->SetColor(colorTransferFunction);
 	volumeProperty->SetScalarOpacity(opacityTransferFunction);
	renderer->SetBackground(0, 0, 0);
	BlackButton->setChecked(true);
	configureButtons(min, max);
	insertRow(min, 255, 255, 255, 0);
	insertRow(max, 255, 255, 255, 0);
	insertRow(mean, 255, 0, 0, 100);
}

void Render::changeRenderFunction() {
  double range[2];
  image->GetPointData()->GetScalars()->GetRange(range);
  int min = (int) range[0];
  int max = (int) range[1];
	colorTransferFunction = vtkColorTransferFunction::New();
	opacityTransferFunction = vtkPiecewiseFunction::New();
	if(strcmp(ProjectionBox->currentText().toLocal8Bit().constData(), "Average") == 0) {
		changeToAveIPFunction(min, max);
	} else if (strcmp(ProjectionBox->currentText().toLocal8Bit().constData(), "Minimum") == 0) {
		changeToMinIPFunction(min, max);
	} else if (strcmp(ProjectionBox->currentText().toLocal8Bit().constData(), "Median") == 0) {
		changeToMedianIPFunction(min, max);
	} else if (strcmp(ProjectionBox->currentText().toLocal8Bit().constData(), "Maximum") == 0) {
		changeToMIPFunction(min, max);
	}
  volumeMapper->SetVolumeRayCastFunction(rayCastFunction);
	declarativeView->update();
}

void Render::changeToBlack()
{
	renderer->SetBackground(0.0, 0.0, 0.0);
	declarativeView->update();
}

void Render::changeToWhite()
{
	renderer->SetBackground(1.0, 1.0, 1.0);
	declarativeView->update();
}

void Render::updateRGBA()
{
	int row = RGBATableWidget->currentRow();
	
	RGBASpinBoxIntensity->setValue(RGBATableWidget->item(row, 0)->text().toInt());
	SpinBoxRed->setValue(RGBATableWidget->item(row, 1)->text().toInt());
	SpinBoxGreen->setValue(RGBATableWidget->item(row, 2)->text().toInt());
	SpinBoxBlue->setValue(RGBATableWidget->item(row, 3)->text().toInt());
	SpinBoxAlpha->setValue(RGBATableWidget->item(row, 4)->text().toInt());
}

void Render::disableButtons()
{
	RGBASpinBoxIntensity->setEnabled(false);
	RGBASliderIntensity->setEnabled(false);
	SliderRed->setEnabled(false);
	SliderGreen->setEnabled(false);
	SliderBlue->setEnabled(false);
	SliderAlpha->setEnabled(false);

	SpinBoxRed->setEnabled(false);
	SpinBoxGreen->setEnabled(false);
	SpinBoxBlue->setEnabled(false);
	SpinBoxAlpha->setEnabled(false);
	CreateRGBA->setEnabled(false);
	RemoveRGBA->setEnabled(false);
	BlackButton->setEnabled(false);
	WhiteButton->setEnabled(false);
	ProjectionBox->setEnabled(false);
	LODSlider->setEnabled(false);
	AnimationButton->setEnabled(false);
}

void Render::setCurrentFile(const QString &fileName)
{
  currentFileName = fileName;
  setWindowFilePath(fileName);
	declarativeView->update();
}

void Render::AddRGBAPoint()
{
	int intensity, red, green, blue, alpha;
	red = SpinBoxRed->value();
	green = SpinBoxGreen->value();
	blue = SpinBoxBlue->value();
	intensity = RGBASpinBoxIntensity->value();
	alpha = SpinBoxAlpha->value();

	colorTransferFunction->AddRGBPoint(intensity, (red/255.0), (green/255.0), (blue/255.0));
	opacityTransferFunction->AddPoint(intensity, (alpha/100.0));

	insertRow(intensity, red, green, blue, alpha);

	declarativeView->update();
}

void Render::RemoveRGBAPoint()
{
	int intensity = RGBASpinBoxIntensity->value();

	colorTransferFunction->RemovePoint(intensity);
	opacityTransferFunction->RemovePoint(intensity);

	for(int i = 0; i < RGBATableWidget->rowCount(); i++) {
		if(RGBATableWidget->item(i, 0)->text().toInt() == intensity) {
			RGBATableWidget->removeRow(i);
		}
	}
	declarativeView->update();
}

void Render::fileOpen() {
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open Image"),
			currentFileName,
			tr("Analyze (*.hdr *.img)"),
			&currentFilter);

	int success = 1;

	if (!fileName.isEmpty()) {
		if (currentFilter == "Analyze (*.hdr *.img)") {
			disableButtons();
			success = loadAnalyzeImage(fileName.toLocal8Bit().constData());
		}	else {
			success = 0;
		}

		if (success) {
			setCurrentFile(fileName);
		}
		else {
			warningMessage(tr("Error reading file %1.").arg(fileName),
					tr("Make sure file is of correct type and retry."));
		}
	}
}

void Render::warningMessage(const QString &text, const QString &info)
{
	QMessageBox msgBox;
	msgBox.setText(text);
	msgBox.setInformativeText(info);
	msgBox.setIcon(QMessageBox::Warning);
	msgBox.exec();
}

void Render::configureButtons(int min, int max) 
{
	int window = (max - min)/3.0;

	RGBASliderIntensity->setMinimum(min);
	RGBASliderIntensity->setMaximum(max);
	RGBASliderIntensity->setEnabled(true);

	RGBASpinBoxIntensity->setMinimum(min);
	RGBASpinBoxIntensity->setMaximum(max);
	RGBASpinBoxIntensity->setEnabled(true);

	RGBASliderIntensity->setValue(window);

	SliderRed->setEnabled(true);
	SliderRed->setValue(0);
	SliderGreen->setEnabled(true);
	SliderGreen->setValue(0);
	SliderBlue->setEnabled(true);
	SliderBlue->setValue(0);
	SliderAlpha->setEnabled(true);
	SliderAlpha->setValue(0);

	SpinBoxRed->setEnabled(true);
	SpinBoxGreen->setEnabled(true);
	SpinBoxBlue->setEnabled(true);
	SpinBoxAlpha->setEnabled(true);
	CreateRGBA->setEnabled(true);
	RemoveRGBA->setEnabled(true);

	for(int i =  RGBATableWidget->rowCount(); i >= 0; i--) {
		RGBATableWidget->removeRow(i);
	}

	BlackButton->setEnabled(true);
	WhiteButton->setEnabled(true);
	ProjectionBox->setEnabled(true);
	LODSlider->setEnabled(true);
	AnimationButton->setEnabled(true);
	DegreeSpinBox->setEnabled(true);
}

void Render::insertRow(int intensity, int R, int G, int B, int A)
{
	int row = -1;
	int numberOfRows = RGBATableWidget->rowCount();
	for(int i = 0; i < numberOfRows; i++){
		if(RGBATableWidget->item(i, 0)->text().toInt() == intensity) {
			row = i;
			break;
		}
	}

	if(row == -1) {
		row = numberOfRows;
		RGBATableWidget->setRowCount(numberOfRows + 1);
	}

	QTableWidgetItem *itemIntensity =  new QTableWidgetItem();
	itemIntensity->setData(Qt::DisplayRole, intensity);
	RGBATableWidget->setItem(row, 0, itemIntensity);

	QTableWidgetItem *itemR =  new QTableWidgetItem(QString::number(R));
	RGBATableWidget->setItem(row, 1, itemR);

	QTableWidgetItem *itemG =  new QTableWidgetItem(QString::number(G));
	RGBATableWidget->setItem(row, 2, itemG);

	QTableWidgetItem *itemB =  new QTableWidgetItem(QString::number(B));
	RGBATableWidget->setItem(row, 3, itemB);

	QTableWidgetItem *itemA =  new QTableWidgetItem(QString::number(A));
	RGBATableWidget->setItem(row, 4, itemA);

	RGBATableWidget->sortItems(0, Qt::AscendingOrder);
}

int Render::loadAnalyzeImage(const QString &fileName)
{
	vtkSmartPointer<vtkmsqAnalyzeReader> imageReader =
			vtkSmartPointer<vtkmsqAnalyzeReader>::New();

	vtkSmartPointer<vtkEventQtSlotConnect> connection =
			vtkSmartPointer<vtkEventQtSlotConnect>::New();

	if (imageReader->CanReadFile(fileName.toLocal8Bit().constData()) == 0) {
		return 0;
	}

	imageReader->SetFileName(fileName.toLocal8Bit().constData());
	imageReader->SetImageProperties(properties);
	imageReader->UpdateWholeExtent();

	vtkImageShiftScale *imageShiftScale = vtkImageShiftScale::New();
	if(imageReader->GetDataScalarType() == VTK_CHAR){
		imageShiftScale->SetOutputScalarTypeToUnsignedChar();
	} else if (imageReader->GetDataScalarType() != VTK_UNSIGNED_CHAR && imageReader->GetDataScalarType() != VTK_UNSIGNED_SHORT) {
		imageShiftScale->SetOutputScalarTypeToUnsignedShort();
	}

	imageShiftScale->SetInputConnection(imageReader->GetOutputPort());
	imageShiftScale->Update();

	image = vtkImageData::New();
	image->ShallowCopy(imageShiftScale->GetOutput());

	double range[2];
	image->GetPointData()->GetScalars()->GetRange(range);
	int min = (int) range[0];
	int max = (int) range[1];

	configureButtons(min, max);
	renderer = vtkRenderer::New();
	renderWindow = declarativeView->GetRenderWindow();

	vtkInteractorStyleTrackballCamera *interactorStyle = vtkInteractorStyleTrackballCamera::New();
	interactor = vtkRenderWindowInteractor::New();

  volumeMapper = vtkVolumeRayCastMapper::New();
	volumeProperty = vtkVolumeProperty::New();

  volumeProperty->ShadeOn();
  volumeProperty->SetInterpolationTypeToLinear();

	volumeMapper->SetNumberOfThreads(2);
  volumeMapper->SetInputConnection(imageShiftScale->GetOutputPort());

	changeToMIPFunction(min, max);

	vtkLODProp3D *prop = vtkLODProp3D::New();
	int id = prop->AddLOD(volumeMapper, volumeProperty, 0.0);
	renderer->AddActor(prop);

	highLOD = interactor->GetStillUpdateRate();
	interactor->SetRenderWindow(renderWindow);
	interactor->SetInteractorStyle(interactorStyle);
	interactor->Initialize();

	renderWindow->AddRenderer(renderer);

	declarativeView->update();
	ProjectionBox->setCurrentIndex(0);

	return 1;
}

Render::~Render()
{
	if (properties != NULL) {
		properties->Delete();
	}
}

