/*
 *  Render.h
 *  Tcc
 */

#ifndef _LAYOUT_h
#define _LAYOUT_h

#include <QMainWindow>
#include "ui_Layout.h"

class vtkEventQtSlotConnect;
class vtkColorTransferFunction;
class vtkPiecewiseFunction;
class vtkRenderer;
class vtkLookupTable;
class vtkImageData;
class vtkmsqImageProperties;
class vtkVolumeRayCastFunction;
class vtkVolumeRayCastMapper;
class vtkVolumeProperty;
class vtkRenderWindow;
class vtkRenderWindowInteractor;

class Render : public QMainWindow, public Ui::Layout
{
	Q_OBJECT
public:
	Render();
	~Render();

protected:
	vtkPiecewiseFunction *opacityTransferFunction;
	vtkColorTransferFunction *colorTransferFunction;
	vtkmsqImageProperties *properties;
	vtkImageData *image;
	vtkRenderer *renderer;
	vtkLookupTable *colorMap;
	vtkVolumeRayCastFunction *rayCastFunction;
	vtkVolumeRayCastMapper *volumeMapper;
	vtkVolumeProperty *volumeProperty;
	vtkRenderWindow *renderWindow;
	vtkRenderWindowInteractor *interactor;

	QString currentFileName;
	QString currentFilter;
	QTimer *timer;

	bool isRotating;
	double highLOD;
private:
	int loadAnalyzeImage(const QString &); 
	void warningMessage(const QString &, const QString &);
	void setCurrentFile(const QString &);
	void configureButtons(int min, int max);
	void insertRow(int intensity, int R, int G, int B, int A);
	void disableButtons();
	void changeToAveIPFunction(int min, int max);
	void changeToMIPFunction(int min, int max);
	void changeToMinIPFunction(int min, int max);
	void changeToMedianIPFunction(int min, int max);

private slots:
	void AddRGBAPoint();
	void RemoveRGBAPoint();
	void fileOpen();
	void updateRGBA();
	void changeToBlack();
	void changeToWhite();
	void changeRenderFunction();
	void animation();
	void rotate();
	void changeLOD();
};

#endif
