#include "vtkmsqImageProperties.h"
#include "vtkObjectFactory.h"

//----------------------------------------------------------------------------
vtkCxxRevisionMacro(vtkmsqImageProperties, "0.1");
vtkStandardNewMacro(vtkmsqImageProperties);

static const char *vtkmsqImagePropertiesOrientationString[] = {
  "AXIAL",
  "CORONAL",
  "SAGITTAL",
  NULL
};

//----------------------------------------------------------------------------
vtkmsqImageProperties::vtkmsqImageProperties()
{
  this->EchoTime = NULL;
  this->RepetitionTime = NULL;
  this->Orientation = vtkmsqImageProperties::AXIAL;
  this->Window = 0;
  this->Level = 0;
  this->DirectionCosine[0] = 1;
  this->DirectionCosine[1] = 0;
  this->DirectionCosine[2] = 0;
  this->DirectionCosine[3] = 0;
  this->DirectionCosine[4] = 1;
  this->DirectionCosine[5] = 0;
}

//----------------------------------------------------------------------------
vtkmsqImageProperties::~vtkmsqImageProperties()
{
  this->Clear();
}

//----------------------------------------------------------------------------
void vtkmsqImageProperties::Clear()
{
  this->SetEchoTime(NULL);
  this->SetRepetitionTime(NULL);
}

//----------------------------------------------------------------------------
const char *vtkmsqImageProperties::GetStringFromOrientationType(unsigned int type)
{
  static unsigned int numtypes = 0;
  // find length of table
  if (!numtypes)
    {
    while (vtkmsqImagePropertiesOrientationString[numtypes] != NULL)
      {
      numtypes++;
      }
    }

  if (type < numtypes)
    {
    return vtkmsqImagePropertiesOrientationString[type];
    }

  return NULL;
}

//----------------------------------------------------------------------------
void vtkmsqImageProperties::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os, indent);

  os << "\n" << indent << "Orientation: ";
  os << this->GetStringFromOrientationType(this->Orientation);

  os << "\n" << indent << "EchoTime: ";
  if (this->EchoTime)
    {
    os << this->EchoTime;
    }

  os << "\n" << indent << "RepetitionTime: ";
  if (this->RepetitionTime)
    {
    os << this->RepetitionTime;
    }

  os << indent << "Direction Cosine: (" << this->DirectionCosine[0] << ", " 
     << this->DirectionCosine[1] << ", " << this->DirectionCosine[2] << "), ("
     << this->DirectionCosine[3] << ", " << this->DirectionCosine[4] 
     << ", " << this->DirectionCosine[5] << ")\n";

}

