// .NAME vtkmsqImageProperties - some medical image properties.
// .SECTION Description
// vtkmsqImageProperties is a helper class that can be used by medical
// image readers and applications to encapsulate medical image/acquisition
// properties. 

#ifndef __vtkmsqImageProperties_h
#define __vtkmsqImageProperties_h

#include "vtkObject.h"

class vtkmsqImageProperties : public vtkObject
{
public:
  static vtkmsqImageProperties *New();
  vtkTypeRevisionMacro(vtkmsqImageProperties,vtkObject);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Convenience method to reset all fields to an emptry string/value
  virtual void Clear();

  // Description:
  // Echo Time
  // (Time in ms between the middle of the excitation pulse and the peak of
  // the echo produced)
  // For ex: DICOM (0018,0081) = 105
  vtkSetStringMacro(EchoTime);
  vtkGetStringMacro(EchoTime);

  // Description:
  // Repetition Time
  // The period of time in msec between the beginning of a pulse sequence and
  // the beginning of the succeeding (essentially identical) pulse sequence.
  // For ex: DICOM (0018,0080) = 2040
  vtkSetStringMacro(RepetitionTime);
  vtkGetStringMacro(RepetitionTime);

  // Description:
  // Get the direction cosine (default to 1,0,0,0,1,0)
  vtkSetVector6Macro(DirectionCosine,double);
  vtkGetVector6Macro(DirectionCosine,double);

  // Description: 
  // Get/Set window and level
  vtkSetMacro(Window,double);
  vtkGetMacro(Window,double);

  // Description: 
  // Get/Set window and level
  vtkSetMacro(Level,double);
  vtkGetMacro(Level,double);

 //BTX
  typedef enum {
    AXIAL = 0,
    CORONAL,
    SAGITTAL
  } OrientationType;
  //ETX

  // Description:
  // Get/Set orientation type
  vtkGetMacro(Orientation,int);
  vtkSetClampMacro(Orientation,int,AXIAL,SAGITTAL);

  static const char *GetStringFromOrientationType(unsigned int type);

protected:
  vtkmsqImageProperties();
  ~vtkmsqImageProperties();

  char *EchoTime;
  char *RepetitionTime;
  double DirectionCosine[6];
  double Level;
  double Window;
  int Orientation;

private:
  vtkmsqImageProperties(const vtkmsqImageProperties&); // Not implemented.
  void operator=(const vtkmsqImageProperties&); // Not implemented.
};

#endif
